@@ -54,7 +54,7 @@ __author__: str = "Dominic Davis-Foster"
 __copyright__: str = "2020 Dominic Davis-Foster"
 
 __license__: str = "MIT License"
-__version__: str = "0.7.0"
+__version__: str = "0.8.0"
 __email__: str = "dominic@davis-foster.co.uk"
 
 __all__ = ["setup"]
@@ -114,8 +114,8 @@ def setup(app: Sphinx) -> Dict[str, Any]:
 	app.add_config_value("wikipedia_lang", "en", "env", [str])
 
 	# Asset role
-	app.add_role('asset', assets.asset_role)
-	app.add_config_value('assets_dir', "./assets", 'env', [str])
+	app.add_role("asset", assets.asset_role)
+	app.add_config_value("assets_dir", "./assets", "env", [str])
 	app.add_node(assets.AssetNode, html=(assets.visit_asset_node, assets.depart_asset_node))
 
 	# Setup standalone extensions
@@ -123,6 +123,7 @@ def setup(app: Sphinx) -> Dict[str, Any]:
 	app.setup_extension("sphinx_toolbox.formatting")
 	app.setup_extension("sphinx_toolbox.more_autodoc.autoprotocol")
 	app.setup_extension("sphinx_toolbox.more_autodoc.autotypeddict")
+	app.setup_extension("sphinx_toolbox.more_autodoc.autonamedtuple")
 	app.setup_extension("sphinx_toolbox.source")
 
 	# Hack to get the docutils tab size, as there doesn't appear to be any other way
